##This function is stolen from here: http://lamages.blogspot.com/2013/04/how-to-change-alpha-value-of-colours-in.html
##With some slight modifcations.
add.alpha <- function(col, alpha=0.65){
	apply(
		sapply(col, col2rgb)/255, 
		2, 
		function(x){
			rgb(x[1], x[2], x[3], alpha=alpha)
		}
	)  
}




baseMFA <- function(data,col.des,row.des,norm=T,center=T,scale=T,graphs=T){


	##center scale
	data <- expo.scale(data,center=T,scale=T)

	sub.tables <- vector("list",nrow(col.des))
	for(i in 1:nrow(col.des)){
	
	  sub.tables[[i]] <- data[,which(col.des[i,]==1)]
	  
	}
	names(sub.tables) <- rownames(col.des)
	##step 1: normalize each table to have a variance of 1 -- this is done by dividing by the 
	  ##first singular value of each table.
	if(norm){
		sub.tables.normed <- lapply(sub.tables,function(x){x/svd(x)$d[1]})
	}else{
		sub.tables.normed <- sub.tables
	}
	

	##step 2: but the sub-tables back together into a big table.
	big.normed.mat <- do.call(cbind,sub.tables.normed)
	
	
	global.pca <- epPCA(big.normed.mat,center=F,scale=F,graphs=F,DESIGN=row.des,make_design_nominal=F)
	##table contributions
	ct <- matrix(NA,nrow(col.des),length(global.pca$ExPosition.Data$pdq$Dv))
	for(i in 1:nrow(col.des)){
		ct[i,] <- colSums(global.pca$ExPosition.Data$cj[which(col.des[i,]==1),])
	}
	
	
	##should be able to do sup projections here -- this is correct, not the above.
	sup.blocks <- vector("list",nrow(col.des))
	for(i in 1:length(sub.tables.normed)){
		sup.blocks[[i]] <- supplementalProjection(length(sub.tables.normed) * sub.tables.normed[[i]], global.pca$ExPosition.Data$fj[which(col.des[i,]==1),],Dv=global.pca$ExPosition.Data$pdq$Dv)$f.out
		rownames(sup.blocks[[i]]) <- paste0(names(sub.tables.normed)[i],"_",rownames(sup.blocks[[i]]))
	}	
	
	
	###these are all the basics: everything from the global PCA, table contributions (ct), and the partial projections
	global.pca$ExPosition.Data$ct <- ct
	rownames(global.pca$ExPosition.Data$ct) <- names(sub.tables.normed)
	global.pca$ExPosition.Data$partial.projections <- sup.blocks
	class(global.pca$ExPosition.Data) <- class(global.pca$ExPosition.Data)[-1]
	class(global.pca$Plotting.Data) <- class(global.pca$Plotting.Data)[-1]	
	
	
		##must fix constraints
	global.pca$Plotting.Data$constraints <- minmaxHelper(do.call(rbind,global.pca$ExPosition.Data$partial.projections),global.pca$ExPosition.Data$fj)
	##also, generate some colors for the partial projections.
	ti.cols <- createColorVectorsByDesign(diag(nrow(col.des)),hsv=T,offset=.13)$oc
	partial.projection_ft.cols <- vector("list",nrow(col.des))
	partial.projection_fi.cols <- vector("list",nrow(col.des))	
	for(i in 1:length(partial.projection_ft.cols)){
		partial.projection_ft.cols[[i]] <- rep(add.alpha(ti.cols[i]),nrow(global.pca$ExPosition.Data$partial.projections[[i]]))
		partial.projection_fi.cols[[i]] <- add.alpha(global.pca$Plotting.Data$fi.col)	
	}
	global.pca$Plotting.Data$partial$ft.cols <- partial.projection_ft.cols
	global.pca$Plotting.Data$partial$fi.cols <- partial.projection_fi.cols	
	
	ret.list <- list(analysis=global.pca$ExPosition.Data,plotting=global.pca$Plotting.Data)
	
	if(graphs){
		plot.battery(ret.list,1,2,row.des=row.des,col.des=col.des)
	}
	
	return(ret.list)
}



plot.battery <- function(res,axis.1=1,axis.2=2,row.des=NULL,col.des=NULL){

	prettyScree(res$analysis$eigs)

	prettyPlot(res$analysis$fi,col=res$plotting$fi.col,constraints=res$plotting$constraints,x_axis=axis.1,y_axis=axis.2,pch=20,cex=2,text.cex=1.5,main="Rows")
	

	##by ind colors
	prettyPlot(res$analysis$fi,col=add.alpha(res$plotting$fi.col,0),constraints=res$plotting$constraints,x_axis=axis.1,y_axis=axis.2,pch=20,cex=2,text.cex=1.5,main="Rows -- with partial projections colored by group")
	for(i in 1:length(res$analysis$partial.projections)){
		prettyPlot(res$analysis$partial.projections[[i]],col=res$plotting$partial$fi.cols[[i]],dev.new=F,axes=F,new.plot=F,pch=20)
		for(k in 1:nrow(res$analysis$partial.projections[[i]])){
	    	x.dot <- c(res$analysis$partial.projections[[i]][k,axis.1],res$analysis$fi[k,axis.1])
	    	y.dot <- c(res$analysis$partial.projections[[i]][k,axis.2],res$analysis$fi[k,axis.2])    
	    	points(x.dot,y.dot,col=add.alpha(res$plotting$partial$fi.cols[[i]][k],.4),type="l")
		}
	}
	prettyPlot(res$analysis$fi,col=res$plotting$fi.col,constraints=res$plotting$constraints,x_axis=axis.1,y_axis=axis.2,pch=20,cex=2,text.cex=1.5,dev.new=F,axes=F,new.plot=F)


	##by ind colors
	prettyPlot(res$analysis$fi,col=add.alpha(res$plotting$fi.col,0),constraints=res$plotting$constraints,x_axis=axis.1,y_axis=axis.2,pch=20,cex=2,text.cex=1.5,main="Rows -- with partial projections colored by table")
	for(i in 1:length(res$analysis$partial.projections)){
		prettyPlot(res$analysis$partial.projections[[i]],col=res$plotting$partial$ft.cols[[i]],dev.new=F,axes=F,new.plot=F,pch=20,display_names=F,cex=3)
		for(k in 1:nrow(res$analysis$partial.projections[[i]])){
	    	x.dot <- c(res$analysis$partial.projections[[i]][k,axis.1],res$analysis$fi[k,axis.1])
	    	y.dot <- c(res$analysis$partial.projections[[i]][k,axis.2],res$analysis$fi[k,axis.2])    
	    	points(x.dot,y.dot,col=add.alpha(res$plotting$partial$ft.cols[[i]][k],.4),type="l")
		}
	}
	#prettyPlot(res$analysis$fi,col=res$plotting$fi.col,constraints=res$plotting$constraints,x_axis=axis.1,y_axis=axis.2,pch=20,cex=2,text.cex=1.5,dev.new=F,axes=F,new.plot=F,display_names=F)


	prettyPlot(res$analysis$fj,constraints=minmaxHelper(res$analysis$fj),x_axis=axis.1,y_axis=axis.2,col=createColorVectorsByDesign(t(col.des),hsv=T,offset=.13)$oc,main="Columns -- Colored by Table")
	
	
	prettyPlot(res$analysis$ct,main="Partial Inertias")
	
}